/* Cdigo para calcular la capacidad calorfica con el modelo de Debye.
   Francisco Domnguez-Adame.                                         */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define TMIN  0.0001  /* Temperatura mnima en unidades de theta      */
#define TMAX  2.0     /* Temperatura mxima en unidades de theta      */
#define NT  500       /* Nmero de pasos en temperatura               */
#define EPS    1.0e-6 /* Precisin requerida                          */
#define JMAX  20      /* Nmero mximo de iteraciones en simpson      */
#define FILENAME "DATOS.DAT" /* Archivo de datos                      */
#define PI 3.141592654

double funcion(double x) {
   if(x<1e-10) return 1.; /* Lmite del integrando para x pequeo     */
   if(x>200)   return 0.; /* Lmite del integrando para x grande      */
   return (x*x*exp(x)/(exp(x)-1.)/(exp(x)-1.));
}

double simpson(double a,double b) {
   int i,j=0,n=1;
   double eps=1.e30,jn,j2n,sa=1.e30,sd,sum,c=b-a;
   jn=0.5*(funcion(a)+funcion(b));
   while(eps>EPS) {
      j++;
      if(j==JMAX) {
         printf("Demasiadas iteraciones en simpson\n");
         exit(1);
      }
      sum=0.0;
      for(i=1;i<=n;i++) sum+=funcion(a+c*(float)(2.*(n-i)+1.)/(2.*n));
      j2n=jn+sum;
      sd=c*(2.*j2n-jn)/(3.*n);
      eps=fabs(sd/sa-1);
      sa=sd;
      jn=j2n;
      n*=2;
   }
   return sd;
}

void main(void) {
   double t,dt=(TMAX-TMIN)/(double)NT;
   FILE *out;
   out=fopen(FILENAME,"w");
   for(t=TMIN;t<=TMAX;t+=dt)
      fprintf(out,"%8.6lf    %8.6lf\n",t,t*simpson(0.,1./t));
   fclose(out);
}
