/* Cdigo para el clculo de la contribucin electrnica a la
   capacidad calorfica del cobre a baja temperatura.
   Francisco Domnguez-Adame.                                         */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define N    14         /* Nmero de datos                            */
#define R     1.986     /* Constante molar en J/mol K                 */
#define K     8.62e-5   /* Constante Boltzmann en eV/K                */
#define PI    3.141592654
#define FILENAME  "DATOS.DAT"  /* Archivo de datos                    */

void main(void) {
   float m,c,error_m,error_c,r,d,x_pr,y_pr,aux;
   float t_debye,error_t,e_fermi,error_e;
   int i;
   float t[14]={0.418,0.517,0.787,1.252,1.452,1.968,2.603,
                3.020,3.498,3.579,4.023,4.606,5.301,6.013};
   float c_v[N]={0.000070,0.000087,0.000136,0.000229,0.000275,
                 0.000410,0.000621,0.000809,0.001065,0.001109,
                 0.001407,0.001867,0.002567,0.003464};
   FILE *out;
   out=fopen(FILENAME,"w");
   for(i=0;i<N;i++) {
      c_v[i]/=t[i];      /* Capacidad calorfica/temperatura          */
      t[i]*=t[i];        /* Cuadrado de la temperatura                */
   }
   x_pr=y_pr=0.;         /* Mnimos cuadrados                         */
   for(i=0;i<N;i++) {
      x_pr+=t[i];
      y_pr+=c_v[i];
   }
   x_pr/=(double)N;
   y_pr/=(double)N;
   d=0.;
   for(i=0;i<N;i++)
      d+=(t[i]-x_pr)*(t[i]-x_pr);
   m=0.;                 /* Pendiente de la recta                     */
   for(i=0;i<N;i++)  m+=(t[i]-x_pr)*c_v[i];
   m/=d;
   c=y_pr-m*x_pr;        /* Trmino independiente                     */
   aux=0.;               /* Error en la pendiente de la recta         */
   for(i=0;i<N;i++) aux+=(c_v[i]-m*t[i]-c)*(c_v[i]-m*t[i]-c);
   error_m=sqrt(aux/d/(double) (N-2));
                         /* Error en el trmino independiente         */
   error_c=sqrt((1./(double)N+x_pr*x_pr/d)*aux/(double) (N-2));
   aux=r=0.;             /* Coeficiente de regresin                  */
   for(i=0;i<N;i++) {
      r+=(t[i]-x_pr)*(c_v[i]-y_pr);
      aux+=(c_v[i]-y_pr)*(c_v[i]-y_pr);
   }
   r/=sqrt(d*aux);
   t_debye=PI*pow(2.4*PI*R/m,1./3.);   /* Temperatura de Debye        */
   error_t=t_debye*error_m/m/3.;       /* y su error                  */
   e_fermi=0.5*PI*PI*R*K/c;            /* Energa de Fermi            */
   error_e=e_fermi*error_c/c;          /* y su error                  */
   fprintf(out,"Factor gamma    %5.3e +- %5.0e  ",c,error_c);
   fprintf(out,"cal/K^{-2} mol\n");
   fprintf(out,"Pendiente       %5.3e +- %5.0e  ",m,error_m);
   fprintf(out,"cal/K^{-4} mol\n");
   fprintf(out,"Coeficiente regresion %f \n\n", r);
   fprintf(out,"T Debye %5.1f +- %3.1f K\n",t_debye,error_t);
   fprintf(out,"E Fermi %5.2f +- %3.2f eV\n",e_fermi,error_e);
   fclose(out);
}
