/* Cdigo para calcular el coeficiente  de transmisin de electrones
   en una red finita de tomos en una dimensin as como la relacin
   dispersin en una red infinita.
   Francisco Domnguez-Adame.                                         */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define N       30                   /* Nmero de tomos              */
#define E_MIN    0.1                 /* Energa mnima                */
#define E_MAX   30.0                 /* Energa mxima                */
#define NE    1000                   /* Nmero de pasos en energa    */
#define LAMBDA   1.0                 /* Constante de acoplamiento     */
#define L        1.0                 /* Distancia interatmica        */
#define PI       3.141592654
#define FILENAME  "DATOS.DAT"        /* Archivo de datos              */

void main(void) {
    int i;
    double e,de,q,s,c,aux,r0,r1,r2,i0,i1,i2,tau;
    FILE *out;
    out=fopen(FILENAME,"w");
    de=(E_MAX-E_MIN)/(double)NE;
    for(e=E_MIN;e<=E_MAX;e+=de) {
        q=sqrt(e);
        s=sin(q*L);          /* Variables auxiliares                  */
        c=cos(q*L);
        aux=2.*(c+LAMBDA*s/q);
        r0=1;                /* Condiciones iniciales                 */
        i0=0;
        r1=0.5*aux;
        i1=s-LAMBDA*c/q;
        for(i=2;i<=N;i++) {  /* Calcula el coeficiente de transmisin */
            r2=aux*r1-r0;
            i2=aux*i1-i0;
            r0=r1;
            i0=i1;
            r1=r2;
            i1=i2;
        }
        tau=1./(r2*r2+i2*i2);
        fprintf(out,"%5.4lf   %5.4lf   ",e,log10(tau));
        if(fabs(aux)<2.)     /* Calcula la relacin de dispersin     */
            fprintf(out,"%5.4lf\n",acos(0.5*aux)/L/PI);
        else
            fprintf(out,"\n");
    }
    fclose(out);
}
