/* Cdigo para calcular la estructura de bandas en un potencial 1D.
   Francisco Domnguez-Adame.                                         */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define E_MIN     0.           /* Energa mnima                      */
#define E_MAX     2.           /* Energa mxima                      */
#define NE      500            /* Nmero de pasos de energa          */
#define L         3.141592654  /* Periodo del potencial               */
#define N        50            /* Nmero de pasos de integracin      */
#define V_0       0.2          /* Constante de acoplamiento           */
#define FILENAME  "DATOS.DAT"  /* Nombre del archivo de datos         */
#define PI        3.141592654
#define V(x)      V_0*cos(2.*PI*(x)/L)

float k[NE];                   /* Momentos cristalinos divididos por L*/
float a[N+1];                  /* Vectores auxiliares A y B           */
float b[N+1];

void main(void) {
    float e,de=(E_MAX-E_MIN)/(float)NE,dx=L/(float)(N+1),p,dens,beta;
    int i_e,i_x;
    FILE *out;
    out=fopen(FILENAME,"w");
    for(i_e=0;i_e<=NE;i_e++)    { /* Bucle en energas                */
        e=E_MIN+i_e*de;
        a[0]=2.+dx*dx*(V(dx)-e);  /* Condiciones iniciales            */
        b[0]=1.;
        beta=2.+dx*dx*(V(2.*dx)-e);
        a[1]=beta*a[0]-1.;
        b[1]=beta*b[0];
        for(i_x=2;i_x<=N;i_x++)  { /* Bucle en coordenadas            */
            beta=2.+dx*dx*(V(i_x*dx+dx)-e);
            a[i_x]=beta*a[i_x-1]-a[i_x-2];
            b[i_x]=beta*b[i_x-1]-b[i_x-2];
        }
        p=0.5*(a[N]-b[N-1]);      /* Evalua el polinomio P            */
        if(fabs(p)<1.) k[i_e]=acos(p)/PI;
        else k[i_e]=2.;           /* Si |p|>1, k fuera zona Brillouin */
    }
    for(i_e=1;i_e<NE-1;i_e++) {   /* Imprime los resultados           */
        e=E_MIN+i_e*de;
        if(k[i_e-1]<1. && k[i_e+1]<1.){ /* Energas de la misma banda */
            dens=fabs(k[i_e+1]-k[i_e-1])/(2.*de);
            fprintf(out,"%8.7f  %8.7f  %8.7f\n",k[i_e],e,dens);
        }
    }
    fclose(out);
}
