/* Cdigo para calcular el coeficiente de transmisin de electrones
   en una red de tomos binaria y desordenada en una dimensin.
   Francisco Domnguez-Adame.                                         */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>

#define N      200      /* Nmero de tomos                           */
#define NP      50      /* Nmero de promedios                        */
#define E_MIN    0.01   /* Energa mnima                             */
#define E_MAX   25.0    /* Energa mxima                             */
#define NE     500      /* Nmero de pasos en energa                 */
#define LAMBDA1  1.0    /* Constante de acoplamiento cristal          */
#define LAMBDA2  3.0    /* Constante de acoplamiento impurezas        */
#define C        0.1    /* Concentracin de impurezas                 */
#define L        1.0    /* Distancia interatmica                     */
#define PI       3.141592654
#define NAME "DATOS1.DAT"/* Archivo de datos                          */

/*    Generador de aleatorios en el intervalo [0,1]                   */

#define RAN() ((double)rand()/(double)(RAND_MAX))

double tau[NE+1];       /* Coeficiente de transmisin                 */
float lambda[N+1];      /* Vector con las constantes de acoplamiento  */

void red (void){        /* Genera una cadena desordenada              */
    int i;
    for(i=1;i<N;i++){
       if(RAN()>=C) lambda[i]=LAMBDA1;
       else lambda[i]=LAMBDA2;
    }
}

void main(void) {
    int i,j,k;
    double de=(E_MAX-E_MIN)/(double)NE,q,s,c,r0,r1,r2,i0,i1,i2;
    double aux,auxmas,auxmenos,auxq;
    FILE *out;
    time_t t;
    srand((unsigned)time(&t));
    out=fopen(NAME,"w");
    for(i=1;i<=NE;i++) tau[i]=0.;    /* Inicializa el vector tau      */
    for(i=1;i<=NP;i++){              /* Bucle en promedios            */
       red();
       for(j=1;j<=NE;j++) {          /* Bucle en energas             */
          q=sqrt(E_MIN+(double)j*de);/* Variables auxiliares          */
          s=sin(q*L);
          c=cos(q*L);
          r0=1;                      /* Condiciones iniciales         */
          i0=0;
          r1=c+(lambda[1]/q)*s;
          i1=s-(lambda[1]/q)*c;
          for(k=2;k<=N;k++) {        /* Calcula tau                   */
             aux=lambda[k]/lambda[k-1];/* Variables auxiliares        */
             auxmas=(aux+1.)*c;
             auxmenos=(aux-1.)*s;
             auxq=2.*lambda[k]*s/q;
             r2=(auxmas+auxq)*r1-aux*r0+auxmenos*i1;
             i2=(auxmas+auxq)*i1-aux*i0-auxmenos*r1;
             r0=r1;
             i0=i1;
             r1=r2;
             i1=i2;
          }
          tau[j]-=log(r2*r2+i2*i2)/(double)NP;
       }                           /* Finaliza el bucle en energas   */
    }                              /* Finaliza el bucle en promedios  */
    for(j=1;j<=NE;j++)
       fprintf(out,"%8.6lf   %8.6lf\n",E_MIN+(double)j*de,exp(tau[j]));
    fclose(out);
}
