/* Cdigo para calcular el coeficiente de transmisin en una red
   binaria y desordenada, en funcin de la longitud del sistema.
   Francisco Domnguez-Adame.                                         */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>

#define N     1000      /* Nmero mximo de tomos                    */
#define NP    1000      /* Nmero de promedios                        */
#define E        6.0    /* Energa del electrn incidente             */
#define LAMBDA1  1.0    /* Constante de acoplamiento cristal          */
#define LAMBDA2  3.0    /* Constante de acoplamiento impurezas        */
#define C        0.1    /* Concentracin de impurezas                 */
#define L        1.0    /* Distancia interatmica                     */
#define PI       3.141592654
#define NAME "DATOS2.DAT"/* Archivo de datos                          */
/*    Generador de aleatorios en el intervalo [0,1]                   */
#define RAN() ((double)rand()/(double)(RAND_MAX))

double tau[N+1];        /* Coeficiente de transmisin                 */
float lambda[N+1];      /* Vector con las constantes de acoplamiento  */

void red (void){        /* Genera una cadena desordenada              */
    int i;
    for(i=1;i<N;i++){
       if(RAN()>=C) lambda[i]=LAMBDA1;
       else lambda[i]=LAMBDA2;
    }
}

void main(void) {
    int i,j;
    double q=sqrt(E),s,c,r0,r1,r2,i0,i1,i2;
    double aux,auxmas,auxmenos,auxq;
    FILE *out;    
    time_t t;
    srand((unsigned)time(&t));
    out=fopen(NAME,"w");
    for(i=1;i<=N;i++) tau[i]=0.;     /* Inicializa el vector tau      */
    s=sin(q*L);                      /* Variables auxiliares          */
    c=cos(q*L);
    for(i=1;i<=NP;i++) {             /* Bucle en promedios            */
       red();
       r0=1;                         /* Condiciones iniciales         */
       i0=0;
       r1=c+(lambda[1]/q)*s;
       i1=s-(lambda[1]/q)*c;
       for(j=2;j<=N;j++) {           /* Calcula tau                   */
          aux=lambda[j]/lambda[j-1]; /* Variables auxiliares          */
          auxmas=(aux+1.)*c;
          auxmenos=(aux-1.)*s;
          auxq=2.*lambda[j]*s/q;
          r2=(auxmas+auxq)*r1-aux*r0+auxmenos*i1;
          i2=(auxmas+auxq)*i1-aux*i0-auxmenos*r1;
          r0=r1;
          i0=i1;
          r1=r2;
          i1=i2;
          tau[j]-=log(r2*r2+i2*i2)/(double)NP;
       }
    }                        /* Finaliza el bucle en promedios        */
    for(j=2;j<=N;j++) fprintf(out,"%8.6lf  %8.6lf\n",(double)j,tau[j]);
    fclose(out);
}
