/* Cdigo para calcular la masa efectiva en un modelo Kronig-Penney 3D.
   Francisco Domnguez-Adame.                                         */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define B        1.5           /* Valor inicial de beta               */
#define N_MAX  100             /* Nmero mximo de sumandos           */
#define EPS      0.0001        /* Precisin de las sumas              */

double a(double beta,int l,int m,int n) { /* Calcula a(beta,l,m,n)    */
    double aux;
    aux=sqrt(l*l+m*m+n*n);
    return (exp(-beta*aux)/aux);
}

double b(double beta,int l,int m,int n) { /* Calcula b(beta,l,m,n)    */
    double aux;
    aux=sqrt(l*l+m*m+n*n);
    return exp(-beta*aux);
}

double c(double beta,int l,int m,int n) {  /* Calcula c(beta,l,m,n)   */
    double aux;
    aux=sqrt(l*l+m*m+n*n);
    return (exp(-beta*aux)*aux);
}

double r(double y,int k) {     /* Calcula R_k                         */
    int m,n;
    double suma_par;
    if(k==1) {
        suma_par=6.*a(y,1,0,0)+12.*a(y,1,1,0)+8.*a(y,1,1,1);
        return suma_par;
    }
    suma_par=6.*a(y,k,0,0)+12.*a(y,k,k,0)+8.*a(y,k,k,k);
    for(n=1;n<k;n++)
        suma_par+=24.*(a(y,k,0,n)+a(y,k,k,n));
    for(n=1;n<k;n++)
        for(m=1;m<k;m++)
            suma_par+=24.*a(y,k,m,n);
    return suma_par;
}

double s(double y,int k) {     /* Calcula S_k                         */
    int m,n;
    double suma_par;
    if(k==1) {
        suma_par=6.*b(y,1,0,0)+12.*b(y,1,1,0)+8.*b(y,1,1,1);
        return suma_par;
    }
    suma_par=6.*b(y,k,0,0)+12.*b(y,k,k,0)+8.*b(y,k,k,k);
    for(n=1;n<k;n++)
        suma_par+=24.*(b(y,k,0,n)+b(y,k,k,n));
    for(n=1;n<k;n++)
        for(m=1;m<k;m++)
            suma_par+=24.*b(y,k,m,n);
    return suma_par;
}

double t(double y,int k) {     /* Calcula T_k                         */
    int m,n;
    double suma_par;
    if(k==1) {
        suma_par=6.*c(y,1,0,0)+12.*c(y,1,1,0)+8.*c(y,1,1,1);
        return suma_par;
    }
    suma_par=6.*c(y,k,0,0)+12.*c(y,k,k,0)+8.*c(y,k,k,k);
    for(n=1;n<k;n++)
        suma_par+=24.*(c(y,k,0,n)+c(y,k,k,n));
    for(n=1;n<k;n++)
        for(m=1;m<k;m++)
            suma_par+=24.*c(y,k,m,n);
    return suma_par;
}

double suma(double beta) {
    double sum,aux,eps=1.e30;
    int i=0;
    aux=sum=r(beta,1);
    while(eps>EPS) {
       i++;
       if(i==N_MAX) {
          printf("Demasiados pasos en la funcin suma\n");
          exit(1);
       }
       sum+=r(beta,i);
       eps=fabs(aux-sum);
       aux=sum;
    }
    return sum;
}

double dsuma(double beta) {
    double sum,aux,eps=1.e30;
    int i=0;
    aux=sum=s(beta,1);
    while(eps>EPS) {
       i++;
       if(i==N_MAX) {
          printf("Demasiados pasos en la funcin dsuma\n");
          exit(1);
       }
       sum+=s(beta,i);
       eps=fabs(aux-sum);
       aux=sum;
    }
    return sum;
}

double msuma(double beta) {
    double sum,aux,eps=1.e30;
    int i=0;
    aux=sum=t(beta,1);
    while(eps>EPS) {
       i++;
       if(i==N_MAX) {
          printf("Demasiados pasos en la funcin msuma\n");
          exit(1);
       }
       sum+=t(beta,i);
       eps=fabs(aux-sum);
       aux=sum;
    }
    return sum;
}

void main(void) {
    int i=0;
    double beta0=B,beta1,m,eps=1.e30;
    while (eps>EPS) {
       i++;
       if(i==N_MAX) {
          printf("Demasiados pasos en el mtodo de Newton\n");
          exit(1);
       }
       beta1=beta0+(suma(beta0)-beta0)/(dsuma(beta0)+1.);
       eps=fabs(beta1-beta0);
       beta0=beta1;
    }
    beta0=beta1;
    printf("La raz buscada es %8.6lf\n",beta0);
    m=(3./beta0)*(1.+dsuma(beta0))/msuma(beta0);
    printf("La masa efectiva es %8.6lf m\n",m);
}
