/* Cdigo para el clculo de la estructura de bandas en el potencial
   de Mathieu mediante un mtodo variacional.
   Francisco Domnguez-Adame.                                         */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define V0   -2.0       /* Constante del potencial coseno             */
#define V1   -3.0       /* Potencial constante de fondo               */
#define AMIN  0.66      /* Valor mnimo del prametro a               */
#define AMAX  0.92      /* Valor mximo del prametro a               */
#define DK    0.001     /* Incremento del vector recproco            */
#define EPS   0.0001    /* Precisin del mnimo de energa            */
#define NMAX 50         /* Nmero mximo de iteraciones               */
#define PI    3.141592654
#define FILENAME  "DATOS.DAT" /* Archivo de datos                     */

double energia(double a,double mu) {
    double aux1,aux2;
    aux1=V0*exp(-0.5/a);
    aux2=exp(-0.5*a*PI*PI);
    return (a+V1+aux1+mu*aux2*(a-a*a*PI*PI+V1-aux1))/(1.+mu*aux2);
}

double minimo(double mu) {
   double amin=AMIN,amax=AMAX,da=1e20;
   int i=0;
   while(da>EPS) {
      i++;
      if(i==NMAX) {
         puts("Demasiadas iteraciones en la funcin minimo\n");
         exit(1);
      }
      da=(amax-amin)/3.;
      if(energia(amin+da,mu)<energia(amax-da,mu)) amax-=da;
      else amin+=da;
   }
   return (0.5*(amax+amin));
}

void main(void) {
   double aux,k,dk=DK;
   FILE *out;
   out=fopen(FILENAME,"w");
   for(k=0.;k<=1.;k+=dk) {
      aux=minimo(2.*cos(PI*k));
      fprintf(out,"%8.6lf  %8.6lf  ",k,aux);
      fprintf(out,"%8.6lf\n",energia(aux,2.*cos(PI*k)));
   }
   fclose(out);
}
