/* Cdigo para el clculo de minibandas en superredes semiconductoras.
   Francisco Domnguez-Adame                                          */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define MA          0.067         /* Masa efectiva en GaAs            */
#define MB          0.096         /* Masa efectiva en AlGaAs          */
#define V           0.25          /* Altura de las barreras en eV     */
#define A          30.0           /* Anchura de pozos en Amstrongs    */
#define BMIN       15.0           /* Anchura mnima barreras en eV    */
#define BMAX      100.0           /* Anchura mxima barreras en eV    */
#define NB        500             /* Nmero de pasos en barreras      */
#define EMIN        0.01          /* Energa mnima en eV             */
#define EMAX        0.24          /* Energa mxima en eV             */
#define NE       2000             /* Numero de pasos en energas      */
#define RA       MA*0.2625        /* 2m^{*}/hbar^2 eV^{-1}A^{-2} en A */
#define RB       MB*0.2625        /* 2m^{*}/hbar^2 eV^{-1}A^{-2} en B */
#define FILENAME "DATOS.DAT"

void main(void) {
   double k,g,de,e,b,db,aux,espectro,tmp;
   FILE *out;
   out=fopen(FILENAME,"w");
   de=(double)(EMAX-EMIN)/NE;
   db=(double)(BMAX-BMIN)/NB;
   for(b=BMIN;b<=BMAX;b+=db) {
      tmp=-1.;
      fprintf(out,"%4.1lf  ",b);
      for(e=EMIN;e<=EMAX;e+=de) {
         k=sqrt(RA*e);
         g=sqrt(RB*(V-e));
         aux=MB*k/MA/g;
         aux=0.5*(aux-1./aux);
         /* La variable espectro es positiva si la energa est       */
         /* dentro de una minibanda y negativa en caso contrario      */
         espectro=1.-fabs(cos(k*A)*cosh(g*b)-aux*sinh(g*b)*sin(k*A));
            /* Imprime cada vez que espectro cambia de signo          */
         if(tmp*espectro<0.)
            fprintf(out,"%9.7lf  ",e);
         tmp=espectro;
      }
      fprintf(out,"\n");
   }
   fclose(out);
}
