/* Cdigo para el clculo de las minibandas en superredes de Fibonacci.
   Francisco Domnguez-Adame                                          */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define N          13             /* Nmero de barreras               */
#define MA          0.067         /* Masa efectiva en GaAs            */
#define MB          0.096         /* Masa efectiva en GaAlAs          */
#define V           0.25          /* Altura de barrera (eV)           */
#define A1         32.0           /* Anchura de pozos estrechos (eV)  */
#define A2         35.0           /* Anchura de pozos anchos (eV)     */
#define B          32.0           /* Anchura de las barreras (A)      */
#define E_MIN       0.08          /* Energa mnima (eV)              */
#define E_MAX       0.15          /* Energa mxima (eV)              */
#define NE        500             /* Nmero de pasos en energas      */
#define RA       MA*0.2625        /* 2m^{*}/hbar^2 eV^{-1}A^{-2} en A */
#define RB       MB*0.2625        /* 2m^{*}/hbar^2 eV^{-1}A^{-2} en B */
#define FILENAME "DATOS.DAT"

float x[N+1];                     /* Espesores de los pozos cunticos */

void elementos(void) {
   int i;
   float w=0.5*sqrt(5.)-0.5,aux0,aux1;
   for (i=1;i<=N;i++){
      aux0=(float)(i)/w;
      aux1=(float)(i+1)/w;
      x[i]=A2+(A1-A2)*(floor(aux1)-floor(aux0)-1.);
   }
}

void main(void) {
   int n;
   double k,g,de=(double)(E_MAX-E_MIN)/NE,e,aux,r0,r1,r2,i0,i1,i2;
   double cn,sn,ra,rd,ia,id,par1,par2;
   FILE *out;
   out=fopen(FILENAME,"w");
   elementos();
   for(e=E_MIN;e<=E_MAX;e+=de) {
      k=sqrt(RA*e);
      g=sqrt(RB*(V-e));
      aux=0.5*(MB*k/MA/g-MA*g/MB/k);
      ra=cosh(g*B)*cos(k*x[1])-aux*sinh(g*B)*sin(k*x[1]);
      ia=cosh(g*B)*sin(k*x[1])+aux*sinh(g*B)*cos(k*x[1]);
      r0=1.;
      r1=ra;
      i0=0.;
      i1=ia;
      for(n=2;n<=N;n++) {
         cn=cos(k*(x[n]-x[n-1]));
         sn=sin(k*(x[n]-x[n-1]));
         rd=cosh(g*B)*cos(k*x[n])-aux*sinh(g*B)*sin(k*x[n]);
         id=cosh(g*B)*sin(k*x[n])+aux*sinh(g*B)*cos(k*x[n]);
         par1=rd+ra*cn+ia*sn;
         par2=id+ra*sn-ia*cn;
         r2=par1*r1-cn*r0-par2*i1+sn*i0;
         i2=par1*i1-cn*i0+par2*r1-sn*r0;
         r0=r1;
         r1=r2;
         i0=i1;
         i1=i2;
         ra=rd;
         ia=id;
      }
      if(fabs(r2)<1.) fprintf(out,"%d   %8.6lf\n",N,e);
      else fprintf(out,"%d   %8.6lf\n",-1,e);
   }
   fclose(out);
}
