/* Cdigo para calcular la imanacin en funcin de la temperatura 
   en materiales ferromagnticos, usando el mtodo de Newton. 
   Francisco Domnguez-Adame                                          */ 
 
#include <stdlib.h> 
#include <stdio.h> 
#include <math.h> 
 
#define M0       1.0           /* Valor inicial de la imanacin       */ 
#define TMIN     0.01          /* Temperatura reducida mnima         */ 
#define TMAX     1.00          /* Temperatura reducida mxima         */ 
#define DT       0.01          /* Paso en temperatura reducida        */ 
#define EPS      0.0001        /* Precisin en el resultado           */ 
#define NMAX   100             /* Mximo nmero de iteraciones        */ 
#define FILENAME  "DATOS.DAT"  /* Nombre del archivo de datos         */ 
 
double r(double x,double t) {  /* Calcula f(x,t)/f'(x,t)              */ 
    double f, df; 
    f=tanh(x/t)-x; 
    df=cosh(x/t); 
    df=1./t/df/df-1.; 
    return (f/df); 
} 
 
void main(void) {  
    int i; 
    double m0,m1,t,dt=DT,eps; 
    FILE *out;               
    out=fopen(FILENAME,"w"); 
    for(t=TMIN;t<=TMAX;t+=dt) { 
       eps=1.e30; 
       m0=M0; 
       i=0; 
       while(eps>EPS) { 
          i++; 
          if(i==NMAX) { 
             printf("Demasiados pasos en el mtodo de Newton\n"); 
             exit(1); 
          } 
          m1=m0-r(m0,t); 
          eps=fabs(m1-m0); 
          m0=m1; 
       } 
       fprintf(out,"%5.4lf   %5.4lf\n",t,m1); 
    } 
} 
