/* Cdigo para calcular la imanacin y la energa en el modelo
   de Ising bidimensional mediante el algoritmo de Metropolis.
   Francisco Domnguez-Adame                                          */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>

#define B        0.1    /* Campo magntico                            */
#define JMIN     0.05   /* Integral de canje mnima                   */
#define JMAX     1.00   /* Integral de canje mxima                   */
#define DJ       0.05   /* Paso en la integral de canje               */
#define NX      40      /* Dimensin de la red segn el eje X         */
#define NY      40      /* Dimensin de la red segn el eje Y         */
#define NP    1000      /* Nmero de promedios                        */
#define NT     100      /* Nmero de pasos de termalizacin           */
#define C        0.5    /* Concentracin inicial de espines S=+1      */
#define NAME "DATOS.DAT"/* Archivo de datos                           */
#define RED  "RED.DAT"  /* Archivo con la posicin de espines S=+1    */
#define INFO  0         /* 1=Escribir RED.DAT, 0=No escribir RED.DAT  */
              /* Generador de nmeros aleatorios en el intervalo [0,1]*/
#define RAN() ((float)rand()/(float)(RAND_MAX))

int s[NX+1][NY+1];      /* Matriz para almacenar los espines          */
float im[NP];           /* Vector con la imanacin por muestreo       */
float em[NP];           /* Vector con la energa por muestreo         */
float razon[5][2];      /* Matriz con la probabilidad de volteo       */

void red (void){        /* Genera la distribucin inicial de espines  */
   int i,j;
   for(i=1;i<=NX;i++){
      for(j=1;j<=NY;j++){
         if(RAN()>C) s[i][j]=-1;
         else s[i][j]=1;
      }
   }
}

void observables(float canje,int n) {
   int i,j,iaux,jaux;
   float suma=0.;
   for(i=1;i<=NY;i++) {
      if(i>1) iaux=i-1;
      else iaux=NY;
      for(j=1;j<=NX;j++) {
         if(j>1) jaux=i-1;
         else jaux=NX;
         im[n]+=s[i][j];
         suma+=s[i][j]*(s[iaux][j]+s[i][jaux]);
      }
   }
   em[n]=-canje*suma-B*im[n];
   im[n]/=(float)(NX*NY);        /* Imanacin por espin               */
   em[n]/=(float)(NX*NY);        /* Energa por espin                 */
}

void main(void) {
   int n,i,j,k,iu,id,ju,jd,espin,aux;
   float ima,ene,dima,dene,canje,dcanje=DJ;
   FILE *out1,*out2;
   time_t t;
   srand((unsigned)time(&t));
   out1=fopen(NAME,"w");
   out2=fopen(RED,"w");
   if(INFO==0) {
      fprintf(out2,"Si desea la configuracin de espines\n");
      fprintf(out2,"debe elegir INFO 1 en el prembulo\n");
   }
   for(canje=JMIN;canje<=JMAX;canje+=dcanje) {
      for(n=0;n<NP;n++) im[n]=em[n]=0.;   /* Inicializa los vectores  */
      for(i=1;i<=5;i++) {                 /* Probabilididad de volteo */
         razon[i][2]=exp(-2.*((2.*i-6)*canje+B));
         razon[i][1]=1./razon[i][2];
      }
      for(n=0;n<NP;n++) {
         red();
         for(k=1;k<=NT;k++) {
            for(i=1;i<=NY;i++) {
               if(i<NY) iu=i+1;
               else iu=1;        /* Condiciones peridicas            */
               if(i>1) id=i-1;
               else id=NY;       /* Condiciones peridicas            */
               for(j=1;j<=NX;j++) {
                  if(j<NX) ju=j+1;
                  else ju=1;     /* Condiciones peridicas            */
                  if(j>1) jd=j-1;
                  else jd=NX;    /* Condiciones peridicas            */
                  espin=s[i][j];
                  aux=s[iu][j]+s[id][j]+s[i][ju]+s[i][jd];
                  if(RAN()<(float)razon[3+aux/2][(3+espin)/2])
                     s[i][j]=-espin;
               }
            }
         }
         observables(canje,n);
      }
      if(INFO==1){/* Escribe la posicin de espines con S=+1 si INFO=1*/
         fprintf(out2,"Canje %f\n",canje);
         for(i=1;i<=NX;i++)
            for(j=1;j<=NY;j++)
               if(s[i][j]==1) fprintf(out2,"%d   %d\n",i,j);
      }
      ima=ene=dima=dene=0.;   /* Clculo de valores promedio y        */
      for(n=0;n<NP;n++) {     /* los correspondientes errores         */
         ima+=im[n]/(float)NP;
         ene+=em[n]/(float)NP;
      }
      for(n=0;n<NP;n++) {
         dima+=(ima-im[n])*(ima-im[n])/(float)NP;
         dene+=(ene-em[n])*(ene-em[n])/(float)NP;
      }
      dima=sqrt(dima);
      dene=sqrt(dene);
      fprintf(out1,"%4.3f   %8.6f   %8.6f   ",canje,ima,ene);
      fprintf(out1,"%8.6f   %8.6f\n",dima,dene);
   }
   fclose(out1);
   fclose(out2);
}
