/* Cdigo para determinar la evolucin temporal de paquetes de onda 
   electrnicos bajo campos elctricos uniformes.
   Francisco Domnguez-Adame                                          */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "complejo.h"

#define N  400               /* Nmero de tomos                      */
#define N0  N/2              /* Posicin inicial                      */
#define TMIN  0              /* Tiempo mximo                         */
#define TMAX  200            /* Tiempo mnimo                         */
#define NT   5000            /* Nmero de pasos temporales            */
#define TSTEP  (double)(TMAX-TMIN)/NT
#define FILENAME   "DATOS.DAT"
#define E  0.100             /* Campo elctrico                       */

complejo ga[N+1];             /* Vector psi en t=k*delta t            */
complejo gd[N+1];             /* Vector psi en t=(k+1)*delta t        */
complejo d[N+1];              /* Elementos de la diagonal de K        */
complejo alfa[N+1],beta[N+1]; /* Vector auxiliar                      */

void elementos(void) {
   int i;
   double a=TSTEP;
   for(i=1;i<=N;i++) d[i]=asigna(E*(i-N0),2./a);
}

void inversion(complejo *d,complejo *r,int n) {
   int i;
   alfa[n]=beta[n]=gd[0]=asigna(0.,0.);
   for(i=n;i>=1;i--) {
      alfa[i-1]=c(asigna(1.,0.),m(d[i],alfa[i]));
      beta[i-1]=c(s(beta[i],r[i]),m(d[i],alfa[i]));
   }
   for(i=0;i<n;i++) gd[i+1]=s(x(alfa[i],gd[i]),beta[i]);
}

void main(void) {
   double temp,modulo,distancia,radio2,tstep=TSTEP;
   int i,j=0,n0=N0;
   FILE *out;
   out=fopen(FILENAME,"w");
   elementos();
   for(i=1;i<=N;i++) ga[i]=asigna(0.,0.);   /* Condiciones iniciales  */
   ga[N0]=asigna(1.,0.);
   temp=TMIN;
   while(temp<TMAX) {
      temp+=tstep;
      inversion(d,ga,N);
      for(i=1;i<=N;i++) {
         gd[i]=m(x(asigna(0.,4./tstep),gd[i]),ga[i]);
         ga[i]=gd[i];
      }
      radio2=0.;
      for(i=1;i<=N;i++) {
         modulo=norma2(gd[i]);
         if(modulo>1e-6) {
            distancia=(double)(i-n0)*(i-n0);
            radio2+=modulo*distancia;
         }
      }
      if((j/50)*50==j) fprintf(out,"%5.4lf   %7.5lf\n",temp,radio2);
      j++;
   }
   fclose(out);
}
