/* Cdigo para el clculo de la reflectividad de rayos X en multicapas.
   Francisco Domnguez-Adame                                          */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "complejo.h"

#define N         100               /* Nmero de capas  	      */
#define D          50.0             /* Espesor de las capas en A      */
#define A_A         0.16            /* Alfa de la capa A	      */
#define A_B         0.001           /* Alfa de la capa B	      */
#define B_A         0.010           /* Beta de la capa A	      */
#define B_B         0.0             /* Beta de la capa B	      */
#define L_MIN      50.0             /* Longitud onda mnima en A      */
#define L_MAX     300.0             /* Longitud onda mxima en A      */
#define NL       2000               /* Nmero pasos en longitud onda  */
#define FILENAME  "DATOS.DAT"

complejo nr[N+2];                         /* ndice de refraccin     */
complejo  d[N+1];                         /* Exponencial de las fases */
complejo  r[N+2];                         /* Reflectividad            */

void elementos(void) {                    /* Genera la secuencia de   */
   int i;                                 /* ndices de refraccin    */
   for(i=1;i<N;i+=2) {
      nr[i]=asigna(1.-0.5*A_B,0.5*B_B);   /* ndice de capas impares  */
      nr[i+1]=asigna(1.-0.5*A_A,0.5*B_A); /* ndice de capas pares    */
   }
   nr[0]=asigna(1.,0.);                   /* La multicapa est en     */
   nr[N+1]=asigna(1.,0.);                 /* vaco: nr[0]=nr[N+1]=1   */
}

void main(void) {
   int n;
   double l,dl=(double)(L_MAX-L_MIN)/NL;
   complejo kappa_0,kappa_1,aux0,aux1,real1=asigna(1.,0.);
   FILE *out;
   out=fopen(FILENAME,"w");
   elementos();
   for(n=1;n<=N+1;n++) r[n]=c(m(nr[n-1],nr[n]),s(nr[n-1],nr[n]));
   for(l=L_MIN;l<=L_MAX;l+=dl) {
      aux0=asigna(0.,12.5664*D/l);
      for(n=1;n<=N;n++) d[n]=cexp(x(aux0,nr[n]));
      aux0=s(r[N],x(r[N+1],d[N]));
      aux1=s(real1,x(r[N],x(r[N+1],d[N])));
      kappa_1=c(aux0,aux1);
      for(n=N;n>=2;n--) {
         aux0=s(r[n-1],x(kappa_1,d[n-1]));
         aux1=s(real1,x(r[n-1],x(kappa_1,d[n-1])));
         kappa_0=c(aux0,aux1);
         kappa_1=kappa_0;
      }
      fprintf(out,"%4.3lf   %8.6lf\n",l,norma2(kappa_0));
   }
   fclose(out);
}
